@@ -2,16 +2,9 @@ from collections.abc import Callable, Iterable
 from typing import Optional
 
 import numpy as np
-from scipy.integrate import quad, romb
-from scipy.special import (
-    exp1,
-    factorial,
-    hankel1,
-    hankel2,
-    lpmv,
-    spherical_jn,
-    spherical_yn,
-)
+from scipy import special
+from scipy.integrate import quad
+from scipy.special import exp1, factorial, lpmv
 
 from osaft import log
 
@@ -89,7 +82,7 @@ def spherical_2_cartesian_vector(
     v_theta: float,
     theta: float,
 ) -> tuple[float, float]:
-    """Transforms velocities in the a Spherical coordinate system
+    """Transforms velocities in the Spherical coordinate system
     `(v_r, v_theta)` of an axisymmetric model to the Cartesian system
     `(v_x, v_z)`. See `here
     <https://gitlab.ethz.ch/goeringc/src-git/-/wikis/Definition-of-the-Coordinate-System>`__
@@ -171,52 +164,34 @@ def clebsch_gordan_coefficient(
 
 def integrate(
     func: Callable[[float], float], lower: float, upper: float,
-    rel_eps: Optional[float] = None,
-) -> float:
-    """ Integrates :attr:`func` between :attr:`lower` and :attr:`upper`. If a
-    :attr:`rel_eps` is passed then the integration is Scipy's QUADPACK
-    implementation `quad()
-    <https://docs.scipy.org/doc/scipy/reference/generated/
-    scipy.integrate.quad.html>`__
-    with estimated relative error rel_eps. Otherwise Scipy's Romberg
-    integration scheme `romb()
-    <https://docs.scipy.org/doc/scipy/reference/generated/scipy.integrate.romb.
-    html?highlight=romb#scipy.integrate.romb>`__ is used.
+    rel_eps: float = 1e-6,
+) -> tuple[float, float]:
+    """ Integrates :attr:`func` between :attr:`lower` and :attr:`upper`
+    Scipy's QUADPACK implementation `quad()
+    <https://docs.scipy.org/doc/scipy/reference/generated/scipy.integrate
+    .quad.html>`__ .
 
-    :param func: function to be integrated. If no rel_eps is passed the
-        function needs to be vectorized.
+    :param func: function to be integrated.
     :param lower: lower integration limit
     :param upper: upper integration limit
     :param rel_eps: estimated relative error of integration
     """
 
-    if rel_eps:
-        # Logging
-        log.debug(
-            'Relative precision goal for integration has been passed.'
-            'Adaptive integration scheme is chosen.',
-        )
-        # Integration
-        integral, abs_err = quad(
-            func, lower, upper, epsabs=0,
-            epsrel=rel_eps, limit=100,
-        )
-        log.debug(
-            f'Integral = {integral},'
-            f'absolute error estimation = {abs_err}\n',
-        )
-        return integral
-    else:
-        # Logging
-        log.debug(
-            'No relative precision goal for integration has been passed.'
-            'Romberg integration scheme is chosen.',
-        )
-        # Integration
-        arr_x = np.linspace(lower, upper, num=129)
-        dx = arr_x[1] - arr_x[0]
-        arr_y = func(arr_x)
-        return romb(arr_y, dx)
+    # Logging
+    log.debug(
+        'Relative precision goal for integration has been passed.'
+        'Adaptive integration scheme is chosen.',
+    )
+    # Integration
+    integral, abs_err = quad(
+        func, lower, upper, epsabs=0,
+        epsrel=rel_eps, limit=200,
+    )
+    log.debug(
+        f'Integral = {integral},'
+        f'absolute error estimation = {abs_err}\n',
+    )
+    return integral, abs_err
 
 
 def integrate_osc(
@@ -229,10 +204,10 @@ def integrate_osc(
 ) -> tuple[float, float]:
     """ Special purpose integrator for oscillating integrals that
     integrates :attr:`func` between :attr:`lower` and  :attr:`upper`. This
-    methods employs Scipy's QUADPACK implementation `quad()
+    method employs Scipy's QUADPACK implementation `quad()
     <https://docs.scipy.org/doc/scipy/reference/generated/
     scipy.integrate.quad.html>`__. Used primarily for boundary layer
-    integration for elastic fluids. The methods creates a list of
+    integration for elastic fluids. The method creates a list of
     uniformly distributes points in a region of interest where local
     difficulties are expected.
     The density of points per viscous wavelength is given by
@@ -296,7 +271,65 @@ def xexpexp1(z: complex, N: int = 20) -> complex:
         return z * exp(z) * exp1(z)
 
 
-class SpecialFunctions:
+class LegendreFunctions:
+    """ This class gathers all reoccurring Legendre Functions necessary for
+     the osaft package. """
+
+    @staticmethod
+    def cos_poly(theta: float, coefficients: np.array) -> np.ndarray:
+        """ Sum of Legendre polynomial with a cosine function as an argument
+
+        :param theta: angle
+        :param coefficients: coefficients of the polynomial
+        """
+        return np.polynomial.legendre.legval(
+            cos(theta), coefficients,
+            tensor=False,
+        )
+
+    @staticmethod
+    def cos_monomial(degree: int, theta: float, coefficient: complex) -> float:
+        """ Returns a single term of Legendre Cosine series
+
+        :param degree: degree of the polynomial to be returned
+        :param theta:  angle
+        :param coefficient: coefficients of the polynomial
+        """
+        return coefficient * lpmv(0, degree, cos(theta))
+
+    @staticmethod
+    def first_cos_poly(theta: float, coefficients: np.array) -> np.ndarray:
+        """ Sum of associate Legendre Polynomial of order one with a cosine
+        as input variable
+
+        :param theta: angle
+        :param coefficients: coefficients of the polynomial
+        """
+        degrees = len(coefficients)
+        return np.sum(
+            [
+                coefficients[n] * lpmv(1, n, cos(theta))
+                for n in range(degrees)
+            ],
+            axis=0,
+        )
+
+    @staticmethod
+    def first_cos_monomial(
+        degree: int, theta: float,
+        coefficient: complex,
+    ) -> float:
+        """ Returns a single term of associate Legendre Cosine series of the
+        first order
+
+        :param degree: degree of the term
+        :param theta:  angle
+        :param coefficient: coefficient of the polynomial
+        """
+        return coefficient * lpmv(1, degree, cos(theta))
+
+
+class BesselFunctions:
     """ This class gathers all reoccurring spherical hankel and bessel
     functions necessary for the osaft package. """
 
@@ -346,27 +379,27 @@ class SpecialFunctions:
 
     @staticmethod
     def besselj(n: int, z: complex) -> complex:
-        """Order :attr:`n` Spherical Bessel function of the first kind
+        """Spherical Bessel function of the first kind of order `n`
 
         :param n: Order
         :param z: argument
         """
-        return spherical_jn(n, z)
+        return special.spherical_jn(n, z)
 
     @classmethod
     def d1_besselj(cls, n: int, z: complex) -> complex:
-        """First derivative of order :attr:`n` Spherical Bessel function of the
-        first kind
+        """First derivative of the spherical Bessel function of the first kind
+        of order `n`
 
         :param n: order
         :param z: argument
         """
-        return spherical_jn(n, z, derivative=True)
+        return special.spherical_jn(n, z, derivative=True)
 
     @classmethod
     def d2_besselj(cls, n: int, z: complex) -> complex:
-        """Second derivative of order :attr:`n` Spherical Bessel function of
-        the first kind
+        """Second derivative of the spherical Bessel function of the first kind
+        of order `n`
 
         :param n: order
         :param z: argument
@@ -378,8 +411,8 @@ class SpecialFunctions:
 
     @classmethod
     def d3_besselj(cls, n: int, z: complex) -> complex:
-        """Third derivative of order :attr:`n` Spherical Bessel function of the
-        first kind
+        """Third derivative of the spherical Bessel function of the first kind
+        of order `n`
 
         :param n: order
         :param z: argument
@@ -391,8 +424,8 @@ class SpecialFunctions:
 
     @classmethod
     def d4_besselj(cls, n: int, z: complex) -> complex:
-        """Fourth derivative of order :attr:`n` Spherical Bessel function of
-        the first kind
+        """Fourth derivative of the spherical Bessel function of the first kind
+        of order `n`
 
         :param n: order
         :param z: argument
@@ -404,27 +437,27 @@ class SpecialFunctions:
 
     @staticmethod
     def bessely(n: int, z: complex) -> complex:
-        """Order :attr:`n` Spherical Bessel function of the second kind
+        """Spherical Bessel function of the second kind of order `n`
 
         :param n: order
         :param z: argument
         """
-        return spherical_yn(n, z)
+        return special.spherical_yn(n, z)
 
     @classmethod
     def d1_bessely(cls, n: int, z: complex) -> complex:
-        """First derivative of order :attr:`n` Spherical Bessel function of
-        the second kind
+        """First derivative of the spherical Bessel function of the second
+        kind of order `n`
 
         :param n: order
         :param z: argument
         """
-        return spherical_yn(n, z, True)
+        return special.spherical_yn(n, z, True)
 
     @classmethod
     def d2_bessely(cls, n: int, z: complex) -> complex:
-        """Second derivative of order :attr:`n` Spherical Bessel function of
-        the second kind
+        """Second derivative of the spherical Bessel function of the second
+        kind of order `n`
 
         :param n: order
         :param z: argument
@@ -436,8 +469,8 @@ class SpecialFunctions:
 
     @classmethod
     def d3_bessely(cls, n: int, z: complex) -> complex:
-        """Third derivative of order :attr:`n` Spherical Bessel function of
-        the second kind
+        """Third derivative of the spherical Bessel function of the second
+        kind of order `n`
 
         :param n: order
         :param z: argument
@@ -449,8 +482,8 @@ class SpecialFunctions:
 
     @classmethod
     def d4_bessely(cls, n: int, z: complex) -> complex:
-        """Fourth derivative of order :attr:`n` Spherical Bessel function of
-        the second kind
+        """Fourth derivative of the spherical Bessel function of the second
+        kind of order `n`
 
         :param n: order
         :param z: argument
@@ -462,18 +495,17 @@ class SpecialFunctions:
 
     @staticmethod
     def hankelh1(n: int, z: complex) -> complex:
-        """Order :attr:`n` Spherical Hankel function of the first kind
+        """Spherical Hankel function of the first kind of order `n`
 
         :param n: order
         :param z: argument
         """
-        return sqrt(pi / (2 * z)) * hankel1(n + 0.5, z)
+        return sqrt(pi / (2 * z)) * special.hankel1(n + 0.5, z)
 
     @classmethod
     def d1_hankelh1(cls, n: int, z: complex) -> complex:
-        """First derivative of order :attr:`n` Spherical Hankel function of the
-        first
-        kind
+        """First derivative of the spherical Hankel function of the first
+        kind of order `n`
 
         :param n: order
         :param z: argument
@@ -482,8 +514,8 @@ class SpecialFunctions:
 
     @classmethod
     def d2_hankelh1(cls, n: int, z: complex) -> complex:
-        """Second derivative of order :attr:`n` Spherical Hankel function of
-        the first kind
+        """Second derivative of the spherical Hankel function of the first
+        kind of order `n`
 
         :param n: order
         :param z: argument
@@ -495,8 +527,8 @@ class SpecialFunctions:
 
     @classmethod
     def d3_hankelh1(cls, n: int, z: complex) -> complex:
-        """Third derivative of order :attr:`n` Spherical Hankel function of
-        the first kind
+        """Third derivative of the spherical Hankel function of the first
+        kind of order `n`
 
         :param n: order
         :param z: argument
@@ -508,8 +540,8 @@ class SpecialFunctions:
 
     @classmethod
     def d4_hankelh1(cls, n: int, z: complex) -> complex:
-        """Fourth derivative of order :attr:`n` Spherical Hankel function of
-        the first kind
+        """Fourth derivative of the spherical Hankel function of the first
+        kind of order `n`
 
         :param n: order
         :param z: argument
@@ -521,8 +553,8 @@ class SpecialFunctions:
 
     @classmethod
     def d5_hankelh1(cls, n: int, z: complex) -> complex:
-        """Fourth derivative of order :attr:`n` Spherical Hankel function of
-        the first kind
+        """Fifth derivative of the spherical Hankel function of the first
+        kind of order `n`
 
         :param n: order
         :param z: argument
@@ -534,18 +566,17 @@ class SpecialFunctions:
 
     @staticmethod
     def hankelh2(n: int, z: complex) -> complex:
-        """Order :attr:`n` Spherical Hankel function of the second kind
+        """Spherical Hankel function of the second kind of order `n`
 
         :param n: order
         :param z: argument
         """
-        return sqrt(pi / (2 * z)) * hankel2(n + 0.5, z)
+        return sqrt(pi / (2 * z)) * special.hankel2(n + 0.5, z)
 
     @classmethod
     def d1_hankelh2(cls, n: int, z: complex) -> complex:
-        """First derivative of order :attr:`n` Spherical Hankel function of the
-        second
-        kind
+        """First derivative of the spherical Hankel function of the second
+        kind of order `n` second kind
 
         :param n: order
         :param z: argument
@@ -554,8 +585,8 @@ class SpecialFunctions:
 
     @classmethod
     def d2_hankelh2(cls, n: int, z: complex) -> complex:
-        """Second derivative of order :attr:`n` Spherical Hankel function of
-        the second kind
+        """Second derivative of the spherical Hankel function of the second
+        kind of order `n` second kind
 
         :param n: order
         :param z: argument
@@ -567,8 +598,8 @@ class SpecialFunctions:
 
     @classmethod
     def d3_hankelh2(cls, n: int, z: complex) -> complex:
-        """Third derivative of order :attr:`n` Spherical Hankel function of
-        the second kind
+        """Third derivative of the spherical Hankel function of the second
+        kind of order `n` second kind
 
         :param n: order
         :param z: argument
@@ -580,8 +611,8 @@ class SpecialFunctions:
 
     @classmethod
     def d4_hankelh2(cls, n: int, z: complex) -> complex:
-        """Fourth derivative of order :attr:`n` Spherical Hankel function of
-        the second kind
+        """Fourth derivative of the spherical Hankel function of the second
+        kind of order `n` second kind
 
         :param n: order
         :param z: argument
@@ -593,8 +624,8 @@ class SpecialFunctions:
 
     @classmethod
     def d5_hankelh2(cls, n: int, z: complex) -> complex:
-        """Fourth derivative of order :attr:`n` Spherical Hankel function of
-        the second kind
+        """Fifth derivative of the spherical Hankel function of the second
+        kind of order `n` second kind
 
         :param n: order
         :param z: argument
@@ -606,78 +637,74 @@ class SpecialFunctions:
 
     @classmethod
     def adaptive_derivative_besselj(
-            cls, n: int, z: complex, i: int,
+            cls, n: int, z: complex, i: int = 0,
     ) -> complex:
-        """ Returns the value of the :attr:`i`-th derivative of the spherical
-        Bessel function of the first kind of order :attr:`n`. The coefficients
-        for the derivative are calculated at runtime. There is no restriction
-        to the magnitude of :attr:`i` other than it must be greater than `1`.
+        """Returns the value of the `i`-th derivative of the spherical
+        Bessel function of the first kind of order `n`. The
+        coefficients for the derivative are calculated at runtime.
 
         :param n: order of spherical Bessel function
         :param z: argument of the function
         :param i: i-th derivative is computed
         """
-        return cls.__adaptive_derivative(cls.besselj, i, n, z)
+        return cls._adaptive_derivative(cls.besselj, i, n, z)
 
     @classmethod
     def adaptive_derivative_bessely(
             cls, n: int, z: complex, i: int,
     ) -> complex:
-        """ Returns the value of the :attr:`i`-th derivative of the spherical
-        Bessel function of the second kind of order :attr:`n`. The coefficients
-        for the derivative are calculated at runtime.  There is no restriction
-        to the magnitude of :attr:`i` other than it must be greater than `1`.
+        """Returns the value of the `i`-th derivative of the spherical
+        Bessel function of the second kind of order `n`. The coefficients
+        for the derivative are calculated at runtime.
 
         :param n: order of spherical Hankel function
         :param z: argument of the function
-        :param i: i-th derivative is computed
+        :param i: i-th derivative is computed at runtime.
         """
-        return cls.__adaptive_derivative(cls.bessely, i, n, z)
+        return cls._adaptive_derivative(cls.bessely, i, n, z)
 
     @classmethod
     def adaptive_derivative_hankelh1(
             cls, n: int, z: complex, i: int,
     ) -> complex:
-        """ Returns the value of the :attr:`i`-th derivative of the spherical
-        Hankel function of order :attr:`n`. The coefficients for the derivative
-        are calculated at runtime.  There is no restriction to the magnitude of
-        :attr:`i` other than it must be greater than `1`.
+        """Returns the value of the `i`-th derivative of the spherical
+        Hankel function of order `n`. The coefficients for the derivative
+        are calculated at runtime.
 
         :param n: order of spherical Hankel function
         :param z: argument of the function
         :param i: i-th derivative is computed
         """
-        return cls.__adaptive_derivative(cls.hankelh1, i, n, z)
+        return cls._adaptive_derivative(cls.hankelh1, i, n, z)
 
     @classmethod
     def adaptive_derivative_hankelh2(
             cls, n: int, z: complex, i: int,
     ) -> complex:
-        """ Returns the value of the :attr:`i`-th derivative of the spherical
-        Hankel function of order :attr:`n`. The coefficients for the derivative
-        are calculated at runtime.  There is no restriction to the magnitude of
-        :attr:`i` other than it must be greater than `1`.
+        """Returns the value of the `i`-th derivative of the spherical
+        Hankel function of order `n`. The coefficients for the derivative
+        are calculated at runtime.
 
         :param n: order of spherical Hankel function
         :param z: argument of the function
         :param i: i-th derivative is computed
         """
-        return cls.__adaptive_derivative(cls.hankelh2, i, n, z)
+        return cls._adaptive_derivative(cls.hankelh2, i, n, z)
 
     @classmethod
-    def __adaptive_derivative(
+    def _adaptive_derivative(
         cls, func: Callable[[int, complex], complex],
         i: int, n: int, z: complex,
     ) -> complex:
         out = 0
         for j in np.arange(i + 1):
-            out += cls.__adaptive_coefficient(
+            out += cls._adaptive_coefficient(
                 i, j, n, z,
             ) * func(n + j, z)
         return out
 
     @classmethod
-    def __adaptive_coefficient(
+    def _adaptive_coefficient(
         cls, i: int, j: int,
         n: int, z: complex,
     ) -> complex:
@@ -689,69 +716,11 @@ class SpecialFunctions:
             out = n / z
         else:
             out = (n + 2 * j + 1 - i) / z
-            out *= cls.__adaptive_coefficient(i - 1, j, n, z)
-            out -= cls.__adaptive_coefficient(i - 1, j - 1, n, z)
+            out *= cls._adaptive_coefficient(i - 1, j, n, z)
+            out -= cls._adaptive_coefficient(i - 1, j - 1, n, z)
 
         return out
 
 
-class LegendreFunctions:
-    """ This class gathers all reoccurring Legendre Functions necessary for
-     the osaft package. """
-
-    @staticmethod
-    def cos_poly(theta: float, coefficients: np.array) -> np.ndarray:
-        """ Sum of Legendre polynomial with a cosine function as an argument
-
-        :param theta: angle
-        :param coefficients: coefficients of the polynomial
-        """
-        return np.polynomial.legendre.legval(
-            cos(theta), coefficients,
-            tensor=False,
-        )
-
-    @staticmethod
-    def cos_monomial(degree: int, theta: float, coefficient: complex) -> float:
-        """ Returns a single term of Legendre Cosine series
-
-        :param degree: degree of the polynomial to be returned
-        :param theta:  angle
-        :param coefficient: coefficients of the polynomial
-        """
-        return coefficient * lpmv(0, degree, cos(theta))
-
-    @staticmethod
-    def first_cos_poly(theta: float, coefficients: np.array) -> np.ndarray:
-        """ Sum of associate Legendre Polynomial of order one with a cosine
-        as input variable
-
-        :param theta: angle
-        :param coefficients: coefficients of the polynomia
-        """
-        degrees = len(coefficients)
-        return np.sum(
-            [
-                coefficients[n] * lpmv(1, n, cos(theta))
-                for n in range(degrees)
-            ],
-            axis=0,
-        )
-
-    @staticmethod
-    def first_cos_monomial(
-        degree: int, theta: float,
-        coefficient: complex,
-    ) -> float:
-        """ Returns a single term of associate Legendre Cosine series of the
-        first order
-
-        :param degree: degree of the term
-        :param theta:  angle
-        :param coefficient: coefficient of the polynomial
-        """
-        return coefficient * lpmv(1, degree, cos(theta))
-
-
 if __name__ == '__main__':
     pass
