@@ -38,7 +38,7 @@ class ARF(ScatteringField, BaseARF):
     :param eta_f: shear viscosity [Pa s]
     :param zeta_f: bulk viscosity [Pa s]
     :param p_0: Pressure amplitude of the field [Pa]
-    :param wave_type: Type of wave, travel(l)ing or standing
+    :param wave_type: Type of wave, traveling or standing
     :param position: Position in the standing wave field [rad]
     :param small_particle_limit: using the limiting cases for the ARF
     :param small_viscous_boundary_layer: :math:`x_v \\ll 1`
